import { Grid} from "@mui/material";
import React, { memo, useEffect, useRef} from "react";


const CalendarTask = ({setCurrentTask, handleOpen, task, line, setCurrentLine, currentLine, currentTask,  dragTaskHandler, hour}) => {

  const [color, setColor] = useState('')
  useEffect(() => {
    if (task.priority) {
      if (task.priority === 'A') setColor('rgb(32, 138, 250)')
      if (task.priority === 'B') setColor('lightgreen')
      if (task.priority === 'C') setColor('yellow')
    } else {
      setColor('rgb(171, 157, 157);')
    }
  }, [task])

  const ref = useRef(null);
  const refLeft = useRef(null);
  const refRight = useRef(null);

  useEffect(() => {
    const resizeableEle = ref.current;
    const styles = window.getComputedStyle(resizeableEle);
    let width = parseInt(styles.width, 10);
    let x = 0;

    // Right resize
    const onMouseMoveRightResize = (event) => {
      const dx = event.clientX - x;
      x = event.clientX;
      width = width + dx;
      resizeableEle.style.width = `${width}px`;
    };

    const onMouseUpRightResize = (event) => {
      document.removeEventListener("mousemove", onMouseMoveRightResize);
    };

    const onMouseDownRightResize = (event) => {
      x = event.clientX;
      resizeableEle.style.left = styles.left;
      resizeableEle.style.right = null;
      document.addEventListener("mousemove", onMouseMoveRightResize);
      document.addEventListener("mouseup", onMouseUpRightResize);
    };

    // Left resize
    const onMouseMoveLeftResize = (event) => {
      const dx = event.clientX - x;
      x = event.clientX;
      width = width - dx;
      resizeableEle.style.width = `${width}px`;
    };

    const onMouseUpLeftResize = (event) => {
      document.removeEventListener("mousemove", onMouseMoveLeftResize);
    };

    const onMouseDownLeftResize = (event) => {
      x = event.clientX;
      resizeableEle.style.right = styles.right;
      resizeableEle.style.left = null;
      document.addEventListener("mousemove", onMouseMoveLeftResize);
      document.addEventListener("mouseup", onMouseUpLeftResize);
    };

    // Add mouse down event listener
    const resizerRight = refRight.current;
    resizerRight.addEventListener("mousedown", onMouseDownRightResize);
    const resizerLeft = refLeft.current;
    resizerLeft.addEventListener("mousedown", onMouseDownLeftResize);

    return () => {
      resizerRight.removeEventListener("mousedown", onMouseDownRightResize);
      resizerLeft.removeEventListener("mousedown", onMouseDownLeftResize);
    };
  }, []);

  const onClickTaskHandler = (e, task) => {
    e.stopPropagation(); 
    setCurrentTask(task); 
    handleOpen(e)
  }
  
  const dragLeaveHandler = (e) => {
    e.target.style.boxShadow = 'none'
  }
  const dragStartHandler = (e, line, task) => {
    setCurrentLine()
    setCurrentTask(task); 
  }
  const dragEndHandler = (e) => {
    e.target.style.boxShadow = 'none'
  }
  
  return (<>
    <Grid  
        draggable={true} 
        ref={ref}
        onDragLeave={(e)=>{dragLeaveHandler(e)}}
        onDragStart={(e)=>{dragStartHandler(e, line, task)}}
        onDragEnd={(e)=>{dragEndHandler(e)}}
        sx={{ position: 'relative', height: '30px', backgroundColor: color, whiteSpace: 'nowrap', overflow: 'hidden', textOverflow: 'ellipsis', borderRadius: '10px', margin: '5px 10px', display: 'flex', justifyContent: 'flex-start', alignItems: 'center', paddingLeft: '5px', zIndex: '5'}} 
        onClick={(e)=>{onClickTaskHandler(e, task)}}
      >
      <div ref={refLeft} style={{cursor: 'col-resize',height: '100%', left: '0',top: '0', width: '5px', backgroundColor: 'grey'}}></div>
      <span>
        {task.title}
      </span>
      <div ref={refRight} style={{cursor: 'col-resize',height: '100%', right: '0',top: '0', width: '5px', backgroundColor: 'grey'}}></div>
    </Grid>
  </>)
};

export default memo(CalendarTask);